define(["jquery","base/js/namespace","base/js/utils"],((e,t,r)=>(()=>{var o={414:(e,t,r)=>{(()=>{var t={804:e=>{(()=>{"use strict";var t={d:(e,r)=>{for(var o in r)t.o(r,o)&&!t.o(e,o)&&Object.defineProperty(e,o,{enumerable:!0,get:r[o]})},o:(e,t)=>Object.prototype.hasOwnProperty.call(e,t),r:e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})}},r={};t.r(r),t.d(r,{requests:()=>a,urlJoin:()=>o});const o=(...e)=>(e=e.map(((t,r)=>(r&&(t=t.replace(/^\//,"")),r!==e.length-1&&(t=t.replace(/\/$/,"")),t)))).join("/"),n=(window.base_url,{credentials:"same-origin",headers:{"X-CSRFToken":(e=>{let t=null;if(document.cookie&&""!==document.cookie)for(let e of document.cookie.split(";"))if(e.trim().startsWith("_xsrf=")){t=decodeURIComponent(e.substring("_xsrf".length+1));break}return t})()}}),a={get:async(e,t)=>{const r={...n,method:"GET"};return void 0!==t&&(e+="?"+new URLSearchParams(t).toString()),(await fetch(e,r)).json()},post:async(e,t)=>{const r={...n,method:"POST",body:JSON.stringify(t)};return(await fetch(e,r)).json()},put:async(e,t)=>{const r={...n,method:"PUT",body:JSON.stringify(t)};return(await fetch(e,r)).json()},del:async(e,t)=>{const r={...n,method:"DELETE",body:JSON.stringify(t)};return(await fetch(e,r)).json()}};e.exports=r})()}},o={};function n(e){var r=o[e];if(void 0!==r)return r.exports;var a=o[e]={exports:{}};return t[e](a,a.exports,n),a.exports}n.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return n.d(t,{a:t}),t},n.d=(e,t)=>{for(var r in t)n.o(t,r)&&!n.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},n.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),n.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var a={};(()=>{"use strict";n.r(a),n.d(a,{load_help_tab:()=>l});const e=r(404);var t=n.n(e);const o=r(527);var i=n.n(o),s=n(804);function l(){i().notebook_list&&(t()(".tab-content").append(function(){const e=t()("<div/>").attr("id","e2xhelp").addClass("tab-pane"),r=t()("<div/>").attr("id","resources");return r.append(t()("<h4/>").append("Resources")),r.append(t()("<div/>").attr("id","additional-links")),e.append(r),e}()),t()("#tabs").append(t()("<li/>").append(t()("<a/>").attr("href","#e2xhelp").attr("data-toggle","tab").text("Help").on("click",(function(e){window.history.pushState(null,null,"#e2xhelp"),s.requests.get((0,s.urlJoin)(i().notebook_list.base_url,"e2x/help/api/files")).then((e=>{let r=t()("#additional-links");r.empty();let o=t()("<ul/>");e.forEach((function(e){var r,n;o.append((r=(0,s.urlJoin)(i().notebook_list.base_url,"e2x/help/static/",e[1]),n=e[0],t()("<li/>").append(function(e,r,o="_blank"){return t()("<a/>").attr("href",e).attr("target",o).text(r)}(r,n))))})),e.length>0?r.append(o):r.append("There are no additional resources!")}))})))))}})(),e.exports=a})()},527:e=>{"use strict";e.exports=t},30:e=>{"use strict";e.exports=r},404:t=>{"use strict";t.exports=e}},n={};function a(e){var t=n[e];if(void 0!==t)return t.exports;var r=n[e]={exports:{}};return o[e](r,r.exports,a),r.exports}a.n=e=>{var t=e&&e.__esModule?()=>e.default:()=>e;return a.d(t,{a:t}),t},a.d=(e,t)=>{for(var r in t)a.o(t,r)&&!a.o(e,r)&&Object.defineProperty(e,r,{enumerable:!0,get:t[r]})},a.o=(e,t)=>Object.prototype.hasOwnProperty.call(e,t),a.r=e=>{"undefined"!=typeof Symbol&&Symbol.toStringTag&&Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}),Object.defineProperty(e,"__esModule",{value:!0})};var i={};return(()=>{"use strict";a.r(i),a.d(i,{load_ipython_extension:()=>d});var e=a(404),t=a.n(e),r=a(527),o=a.n(r),n=a(30),s=a.n(n),l=a(414);function d(){(0,l.load_help_tab)(),o().notebook_list&&t()("#tabs").append(t()("<li/>").append(t()("<a/>").attr("href",s().url_path_join(o().notebook_list.base_url,"e2x","authoring","app")).attr("target","_blank").text("Authoring")))}})(),i})()));