from .student_exam import load_jupyter_server_extension

_load_jupyter_server_extension = load_jupyter_server_extension

__all__ = [
    "_load_jupyter_server_extension",
    "load_jupyter_server_extension",
]
